soundBank <- {
    "null": ["common/null.wav"]
};

RunWithDelay(-1, function()
{
    foreach (entries in soundBank)
        foreach (line in entries)
            PrecacheScriptSound(line);
})

function PlaySoundToClient(sound)
{
    if (sound in soundBank)
    {
        local randomSound = RandomElement(safeget(soundBank, sound, []));
        if (randomSound)
            EmitSoundOnClient(randomSound, self);
        return randomSound == null;
    }
    return false;
}

function PlaySound(sound, params = {})
{
    if (sound in soundBank)
    {
        local randomSound = RandomElement(safeget(soundBank, sound, []));
        if (randomSound)
            EmitSoundEx(extendtable({
                sound_name = randomSound,
                entity = self,
                channel = 6,
                volume = 1,
                flags =  1,
                sound_level = 150
            }, params));
        return randomSound == null;
    }
    return false;
}

function PlayVoiceLine(sound, params = {})
{
    if (sound in soundBank)
    {
        local randomSound = RandomElement(safeget(soundBank, sound, []));
        if (randomSound)
            EmitSoundEx(extendtable({
                sound_name = randomSound,
                entity = self,
                speaker_entity = self,
                channel = safeget(characterSettings, "mute", false) ? CHAN_USER_BASE + 1 : 2,
                volume = 1,
                sound_level = 150
            }, params));
        return randomSound == null;
    }
    return false;
}

function PlayVoiceLineForAnotherPlayer(listener, sound, params = {})
{
    local randomSound = RandomElement(safeget(soundBank, sound, []));
    if (!randomSound)
        return false;
    //Thanks ficool2 for the idea
    local offset = Vector(999999, 999999, 999999);
    local restore = {};
    foreach (player in GetPlayers())
        if (player != listener)
        {
            restore[player] <- GetPropVector(player, "m_vecViewOffset");
            SetPropVector(player, "m_vecViewOffset", offset);
        }

    EmitSoundEx(extendtable({
        sound_name = randomSound,
        filter_type = RECIPIENT_FILTER_PAS_ATTENUATION,
        channel = safeget(characterSettings, "mute", false) ? CHAN_USER_BASE + 1 : 2,
        entity = self,
        speaker_entity = self,
        volume = 1,
        flags =  1,
        sound_level = 150
    }, params));

    foreach (player, offset in restore)
        SetPropVector(player, "m_vecViewOffset", offset);

    return randomSound == null;
}

function PlayAnnouncerLine(sound, params = {}, positional = false)
{
    if (sound in soundBank)
    {
        local randomSound = RandomElement(safeget(soundBank, sound, []));
        if (randomSound)
            EmitSoundEx(extendtable({
                sound_name = randomSound,
                filter_type = RECIPIENT_FILTER_GLOBAL,
                entity = self,
                channel = CHAN_USER_BASE,
                volume = 1,
                sound_level = positional ? 150 : 0
            }, params));
        return randomSound == null;
    }
    return false;
}

function PlayVoiceLineDelayed(delay, sound, params = {})
{
    RunWithDelay(delay, function() {
        PlayVoiceLine(sound, params);
    })
}

function PlayAnnouncerLineDelayed(delay, sound, params = {}, positional = false)
{
    RunWithDelay(delay, function() {
        PlayAnnouncerLine(sound, params, positional);
    })
}